#! /usr/bin/python
#
##########################################################################
#                                                                        #         
# GeoIP module by Bela, HA5DI                                            #
#                                                                        #         
# Database sourced from http://ip-to-country.webhosting.info             #
#                                                                        #         
# ha5di@freemail.hu                                                      #
#                                                                        #
# 31-jul-2008                                                            #
#                                                                        #
##########################################################################

import	string
import	os.path

version = '1.5 - 19.10.2011'

fn = 'ip-to-country.csv'

###############################################################             
#                                                             #
# Check IP address syntax                                     #
#                                                             #
# Input value:                                                #
#    s = IP address in string 'aaa.bbb.ccc.ddd' format        #
#                                                             #
# Return value:                                               #
#    0 - OK                                                   #
#    1 - not 4 fields provided                                #
#    2 - non integer field                                    #
#    3 - field value out of range                             #
#    4 -                                                      #
#    5 -                                                      #
#                                                             #
###############################################################

def ipchk(s):

    if s[0]==':' and s[1]==':':
	x = s.split(':')
	
	if len(x)<>4:
	    return 4
	    
	if x[0]<>'' or x[1]<>'' or x[2]<>'ffff':
	    return 5
	    
	s = x[3]
	
    w = s.split('.')
    
    if len(w)<>4:
	return 1
    	
    try:
	n = int(w[0])
    except ValueError:
	return 2

    if n<0 or n>255:
	return 3

    try:
        n = int(w[1])
    except ValueError:
	return 2
    
    if n<0 or n>255:
	return 3

    try:
        n = int(w[2])
    except ValueError:
	return 2
    
    if n<0 or n>255:
	return 3

    try:
	n = int(w[3])
    except ValueError:
	return 2
	
    if n<0 or n>255:
	return 3
	
    return 0	


###############################################################             
#                                                             #
# Convert IP address in 'aaa.bbb.ccc.ddd' format to numeric   #
#                                                             #
# Input value:                                                #
#    s = IP address in string 'aaa.bbb.ccc.ddd' format        # 
#                                                             #
# Return value:                                               #
#    n = IP address in numeric format                         #
#                                                             #
# Notes:                                                      #
#    - input is not checked for validity                      #
#                                                             #
###############################################################

def ip2num(s):
    if s[0]==':':
	s = s.split(':')[3]
	
    w = s.split('.')
    n = int(w[0])*256*256*256+int(w[1])*256*256+int(w[2])*256+int(w[3])
    return n


###############################################################             
#                                                             #
# Find country of IP                                          #
#                                                             #
# Input value:                                                #
#    n = IP address in numeric format                         #
#                                                             #
# Return value:                                               #
#    p  - country information                                 #   
#    [] - not found                                           #
#                                                             #
###############################################################

def iplookup(n):
    for p in db:
	if n>=p[0] and n<=p[1]:
	    return [ip2str(p[0]), ip2str(p[1]), p[2], p[3], p[4]]
	
    return []


###############################################################             
#                                                             #
# Convert IP in numeric format to 'aaa.bbb.ccc.ddd' format    #
#                                                             #
# Input value:                                                #
#    n = IP address in numeric format                         #
#                                                             #
# Return value:                                               #
#    s = IP address in string 'aaa.bbb.ccc.ddd' format        #   
#    '' - invalid input                                       #
#                                                             #
###############################################################

def ip2str(n):
    s = ''
    
    if n<0 or n>=256*256*256*256:
	return s
	
    x1 = n%256
    k = n /256
    
    x2 = k%256
    k = k/256
    
    x3 = k%256
    k = k/256
    
    s = '%d.%d.%d.%d' % (k, x3, x2, x1)

    return s

###############################################################             
#                                                             #
# Build IP database from file in memory                       #
#                                                             #
# Input value:                                                #
#    none                                                     #
#                                                             #
# Return value:                                               #
#    0 - OK                                                   #  
#    1 - error                                                #
#                                                             #
###############################################################
                  
def dbload(p):
        
    ap = os.path.abspath(p)
    bn = os.path.basename(p)
    w = ap[:len(ap)-len(bn)]+fn
    
    try:
        fp = open(w)

    except:
	return 1

    while 1:

        p = fp.readline()
    
	if len(p) == 0:
	    break
	
        if p[0]<>'"':
	    continue

        w = p.split(',')
    
	if len(w)<>5:
	    continue
	
        xfrom    = string.atol(w[0].split('"')[1])
        xto      = string.atol(w[1].split('"')[1])
        xcty     = w[2].split('"')[1]
        xcntry   = w[3].split('"')[1]
        xcountry = w[4].split('"')[1]
    
	db.append([xfrom, xto, xcty, xcntry, xcountry])    

    if len(db)<1000:
	return 2
	
    fp.close()
    
    db.append([ip2num('10.0.0.0'), ip2num('10.255.255.255'), '**', '***', 'local network'])
    db.append([ip2num('127.0.0.0'), ip2num('127.255.255.255'), '**', '***', 'localhost'])
    db.append([ip2num('172.16.0.0'), ip2num('172.31.255.255'), '**', '***', 'local network'])
    db.append([ip2num('192.168.0.0'), ip2num('192.168.255.255'), '**', '***', 'local network'])
    db.append([ip2num('169.254.0.0'), ip2num('169.254.255.255'), '**', '***', 'local network'])
    
    return 0
    
###############################################################
#                                                             #
# Module initilalization                                      #
#                                                             #
###############################################################
    
db = []
